@@ -77,7 +77,7 @@ int options_need_geoip_info(const or_options_t *options,
 
 void save_transport_to_state(const char *transport_name,
                              const tor_addr_t *addr, uint16_t port);
-const char *get_bindaddr_for_transport(const char *transport);
+char *get_stored_bindaddr_for_server_transport(const char *transport);
 
 int getinfo_helper_config(control_connection_t *conn,
                           const char *question, char **answer,
