@@ -4,6 +4,83 @@
 /**
  * \file transports.c
  * \brief Pluggable Transports related code.
+ *
+ * \details
+ * Each managed proxy is represented by a <b>managed_proxy_t</b>.
+ * Each managed proxy can support multiple transports.
+ * Each managed proxy gets configured through a multistep process.
+ *
+ * ::managed_proxy_list contains all the managed proxies this tor
+ * instance is supporting.
+ * In the ::managed_proxy_list there are ::unconfigured_proxies_n
+ * managed proxies that are still unconfigured.
+ *
+ * In every run_scheduled_event() tick, we attempt to launch and then
+ * configure the unconfiged managed proxies, using the configuration
+ * protocol defined in the 180_pluggable_transport.txt proposal. A
+ * managed proxy might need several ticks to get fully configured.
+ *
+ * When a managed proxy is fully configured, we register all its
+ * transports to the circuitbuild.c subsystem. At that point the
+ * transports are owned by the circuitbuild.c subsystem.
+ *
+ * When a managed proxy fails to follow the 180 configuration
+ * protocol, it gets marked as broken and gets destroyed.
+ *
+ * <b>In a little more detail:</b>
+ *
+ * While we are serially parsing torrc, we store all the transports
+ * that a proxy should spawn in its <em>transports_to_launch</em>
+ * element.
+ *
+ * When we finish reading the torrc, we spawn the managed proxy and
+ * expect {S,C}METHOD lines from its output. We add transports
+ * described by METHOD lines to its <em>transports</em> element, as
+ * transport_t structs.
+ *
+ * When the managed proxy stops spitting METHOD lines (signified by a
+ * '{S,C}METHODS DONE' message) we register all the transports
+ * collected to the circuitbuild.c subsystem. At this point, the
+ * pointers to transport_t can be transformed into dangling pointers
+ * at any point by the circuitbuild.c subsystem, and so we replace all
+ * transport_t pointers with strings describing the transport names.
+ * We can still go from a transport name to a transport_t using the
+ * fact that each transport name uniquely identifies a transport_t.
+ *
+ * <b>In even more detail, this is what happens when a SIGHUP
+ * occurs:</b>
+ *
+ * We immediately destroy all unconfigured proxies (We shouldn't have
+ * unconfigured proxies in the first place, except when SIGHUP rings
+ * immediately after tor is launched.).
+ *
+ * We mark all managed proxies and transports to signify that they
+ * must be removed if they don't contribute by the new torrc
+ * (we mark using the <b>marked_for_removal</b> element).
+ * We also mark all managed proxies to signify that they might need to
+ * be restarted so that they end up supporting all the transports the
+ * new torrc wants them to support (using the <b>got_hup</b> element).
+ * We also clear their <b>transports_to_launch</b> list so that we can
+ * put there the transports we need to launch according to the new
+ * torrc.
+ *
+ * We then start parsing torrc again.
+ *
+ * Everytime we encounter a transport line using a known pre-SIGHUP
+ * managed proxy, we cleanse that proxy from the removal mark.
+ * We also mark it as unconfigured so that on the next scheduled
+ * events tick, we investigate whether we need to restart the proxy
+ * so that it also spawns the new transports.
+ * If the post-SIGHUP <b>transports_to_launch</b> list is identical to
+ * the pre-SIGHUP one, it means that no changes were introduced to
+ * this proxy during the SIGHUP and no restart has to take place.
+ *
+ * During the post-SIGHUP torrc parsing, we unmark all transports
+ * spawned by managed proxies that we find in our torrc.
+ * We do that so that if we don't need to restart a managed proxy, we
+ * can continue using its old transports normally.
+ * If we end up restarting the proxy, we destroy and unregister all
+ * old transports from the circuitbuild.c subsystem.
  **/
 
 #define PT_PRIVATE
@@ -64,84 +141,6 @@ static smartlist_t *managed_proxy_list = NULL;
 /** Number of still unconfigured proxies. */
 static int unconfigured_proxies_n = 0;
 
-/** "The main idea is:"
-
-    Each managed proxy is represented by a 'managed_proxy_t'.
-    Each managed proxy can support multiple transports.
-    Each managed proxy gets configured through a multistep process.
-
-    'managed_proxy_list' contains all the managed proxies this tor
-    instance is supporting.
-    In the 'managed_proxy_list' there are 'unconfigured_proxies_n'
-    managed proxies that are still unconfigured.
-
-    In every run_scheduled_event() tick, we attempt to launch and then
-    configure the unconfiged managed proxies, using the configuration
-    protocol defined in the 180_pluggable_transport.txt proposal. A
-    managed proxy might need several ticks to get fully configured.
-
-    When a managed proxy is fully configured, we register all its
-    transports to the circuitbuild.c subsystem. At that point the
-    transports are owned by the circuitbuild.c subsystem.
-
-    When a managed proxy fails to follow the 180 configuration
-    protocol, it gets marked as broken and gets destroyed.
-
-    "In a little more technical detail:"
-
-    While we are serially parsing torrc, we store all the transports
-    that a proxy should spawn in its 'transports_to_launch' element.
-
-    When we finish reading the torrc, we spawn the managed proxy and
-    expect {S,C}METHOD lines from its output. We add transports
-    described by METHOD lines to its 'transports' element, as
-    'transport_t' structs.
-
-    When the managed proxy stops spitting METHOD lines (signified by a
-    '{S,C}METHODS DONE' message) we register all the transports
-    collected to the circuitbuild.c subsystem. At this point, the
-    'transport_t's can be transformed into dangling pointers at any
-    point by the circuitbuild.c subsystem, and so we replace all
-    'transport_t's with strings describing the transport names.  We
-    can still go from a transport name to a 'transport_t' using the
-    fact that transport names uniquely identify 'transport_t's.
-
-    "In even more technical detail I shall describe what happens when
-    the SIGHUP bell tolls:"
-
-    We immediately destroy all unconfigured proxies (We shouldn't have
-    unconfigured proxies in the first place, except when SIGHUP rings
-    immediately after tor is launched.).
-
-    We mark all managed proxies and transports to signify that they
-    must be removed if they don't contribute by the new torrc
-    (marked_for_removal).
-    We also mark all managed proxies to signify that they might need
-    to be restarted so that they end up supporting all the transports
-    the new torrc wants them to support (got_hup).
-    We also clear their 'transports_to_launch' list so that we can put
-    there the transports we need to launch according to the new torrc.
-
-    We then start parsing torrc again.
-
-    Everytime we encounter a transport line using a known pre-SIGHUP
-    managed proxy, we cleanse that proxy from the removal mark.
-
-    We also mark it as unconfigured so that on the next scheduled
-    events tick, we investigate whether we need to restart the proxy
-    so that it also spawns the new transports.
-    If the post-SIGHUP 'transports_to_launch' list is identical to the
-    pre-SIGHUP one, it means that no changes were introduced to this
-    proxy during the SIGHUP and no restart has to take place.
-
-    During the post-SIGHUP torrc parsing, we unmark all transports
-    spawned by managed proxies that we find in our torrc.
-    We do that so that if we don't need to restart a managed proxy, we
-    can continue using its old transports normally.
-    If we end up restarting the proxy, we destroy and unregister all
-    old transports from the circuitbuild.c subsystem.
-*/
-
 /** Return true if there are still unconfigured managed proxies. */
 int
 pt_proxies_configuration_pending(void)
@@ -388,7 +387,7 @@ configure_proxy(managed_proxy_t *mp)
   stdout_buf[pos] = '\0';
 
   /* Split up the buffer */
-  lines = smartlist_create();
+  lines = smartlist_new();
   tor_split_lines(lines, stdout_buf, pos);
 
   /* Handle lines. */
@@ -461,7 +460,7 @@ register_server_proxy(managed_proxy_t *mp)
   /* After we register this proxy's transports, we switch its
      mp->transports to a list containing strings of its transport
      names. (See transports.h) */
-  smartlist_t *sm_tmp = smartlist_create();
+  smartlist_t *sm_tmp = smartlist_new();
 
   tor_assert(mp->conf_state != PT_PROTO_COMPLETED);
   SMARTLIST_FOREACH_BEGIN(mp->transports, transport_t *, t) {
@@ -491,7 +490,7 @@ register_client_proxy(managed_proxy_t *mp)
   /* After we register this proxy's transports, we switch its
      mp->transports to a list containing strings of its transport
      names. (See transports.h) */
-  smartlist_t *sm_tmp = smartlist_create();
+  smartlist_t *sm_tmp = smartlist_new();
 
   tor_assert(mp->conf_state != PT_PROTO_COMPLETED);
   SMARTLIST_FOREACH_BEGIN(mp->transports, transport_t *, t) {
@@ -775,7 +774,7 @@ parse_smethod_line(const char *line, managed_proxy_t *mp)
 
   transport_t *transport=NULL;
 
-  items = smartlist_create();
+  items = smartlist_new();
   smartlist_split_string(items, line, NULL,
                          SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, -1);
   if (smartlist_len(items) < 3) {
@@ -806,7 +805,7 @@ parse_smethod_line(const char *line, managed_proxy_t *mp)
     goto err;
   }
 
-  transport = transport_create(&addr, port, method_name, PROXY_NONE);
+  transport = transport_new(&addr, port, method_name, PROXY_NONE);
   if (!transport)
     goto err;
 
@@ -848,7 +847,7 @@ parse_cmethod_line(const char *line, managed_proxy_t *mp)
 
   transport_t *transport=NULL;
 
-  items = smartlist_create();
+  items = smartlist_new();
   smartlist_split_string(items, line, NULL,
                          SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, -1);
   if (smartlist_len(items) < 4) {
@@ -891,7 +890,7 @@ parse_cmethod_line(const char *line, managed_proxy_t *mp)
     goto err;
   }
 
-  transport = transport_create(&addr, port, method_name, socks_ver);
+  transport = transport_new(&addr, port, method_name, socks_ver);
   if (!transport)
     goto err;
 
@@ -913,28 +912,33 @@ parse_cmethod_line(const char *line, managed_proxy_t *mp)
   return r;
 }
 
-/** Return a string containing the address:port that <b>transport</b>
- *  should use. It's the responsibility of the caller to free() the
- *  received string. */
+/** Return the string that tor should place in TOR_PT_SERVER_BINDADDR
+ *  while configuring the server managed proxy in <b>mp</b>. The
+ *  string is stored in the heap, and it's the the responsibility of
+ *  the caller to deallocate it after its use. */
 static char *
-get_bindaddr_for_proxy(const managed_proxy_t *mp)
+get_bindaddr_for_server_proxy(const managed_proxy_t *mp)
 {
-  char *bindaddr = NULL;
-  smartlist_t *string_tmp = smartlist_create();
+  char *bindaddr_result = NULL;
+  char *bindaddr_tmp = NULL;
+  smartlist_t *string_tmp = smartlist_new();
 
   tor_assert(mp->is_server);
 
   SMARTLIST_FOREACH_BEGIN(mp->transports_to_launch, char *, t) {
-    tor_asprintf(&bindaddr, "%s-%s", t, get_bindaddr_for_transport(t));
-    smartlist_add(string_tmp, bindaddr);
+    bindaddr_tmp = get_stored_bindaddr_for_server_transport(t);
+
+    smartlist_add_asprintf(string_tmp, "%s-%s", t, bindaddr_tmp);
+
+    tor_free(bindaddr_tmp);
   } SMARTLIST_FOREACH_END(t);
 
-  bindaddr = smartlist_join_strings(string_tmp, ",", 0, NULL);
+  bindaddr_result = smartlist_join_strings(string_tmp, ",", 0, NULL);
 
   SMARTLIST_FOREACH(string_tmp, char *, t, tor_free(t));
   smartlist_free(string_tmp);
 
-  return bindaddr;
+  return bindaddr_result;
 }
 
 #ifdef MS_WINDOWS
@@ -963,7 +967,7 @@ set_managed_proxy_environment(LPVOID *envp, const managed_proxy_t *mp)
 
   /* A smartlist carrying all the env. variables that the managed
      proxy should inherit. */
-  smartlist_t *envs = smartlist_create();
+  smartlist_t *envs = smartlist_new();
 
   /* Copy the whole environment of the Tor process.
      It should also copy PATH and HOME of the Tor process.*/
@@ -993,7 +997,7 @@ set_managed_proxy_environment(LPVOID *envp, const managed_proxy_t *mp)
     tor_asprintf(&orport_env, "TOR_PT_ORPORT=127.0.0.1:%s",
                  options->ORPort->value);
 
-    bindaddr_tmp = get_bindaddr_for_proxy(mp);
+    bindaddr_tmp = get_bindaddr_for_server_proxy(mp);
     tor_asprintf(&bindaddr_env, "TOR_PT_SERVER_BINDADDR=%s", bindaddr_tmp);
 
     strcpy(extended_env, "TOR_PT_EXTENDED_SERVER_PORT=127.0.0.1:4200");
@@ -1076,7 +1080,7 @@ set_managed_proxy_environment(char ***envp, const managed_proxy_t *mp)
   tor_asprintf(tmp++, "TOR_PT_STATE_LOCATION=%s", state_loc);
   tor_asprintf(tmp++, "TOR_PT_MANAGED_TRANSPORT_VER=1"); /* temp */
   if (mp->is_server) {
-    bindaddr = get_bindaddr_for_proxy(mp);
+    bindaddr = get_bindaddr_for_server_proxy(mp);
 
     /* XXX temp */
     tor_asprintf(tmp++, "TOR_PT_ORPORT=127.0.0.1:%d",
@@ -1113,23 +1117,23 @@ managed_proxy_create(const smartlist_t *transport_list,
   mp->conf_state = PT_PROTO_INFANT;
   mp->is_server = is_server;
   mp->argv = proxy_argv;
-  mp->transports = smartlist_create();
+  mp->transports = smartlist_new();
 
-  mp->transports_to_launch = smartlist_create();
+  mp->transports_to_launch = smartlist_new();
   SMARTLIST_FOREACH(transport_list, const char *, transport,
                     add_transport_to_proxy(transport, mp));
 
   /* register the managed proxy */
   if (!managed_proxy_list)
-    managed_proxy_list = smartlist_create();
+    managed_proxy_list = smartlist_new();
   smartlist_add(managed_proxy_list, mp);
   unconfigured_proxies_n++;
 
   return mp;
 }
 
-/** Register <b>transport</b> using proxy with <b>proxy_argv</b> to
- *  the managed proxy subsystem.
+/** Register proxy with <b>proxy_argv</b>, supporting transports in
+ *  <b>transport_list</b>, to the managed proxy subsystem.
  *  If <b>is_server</b> is true, then the proxy is a server proxy. */
 void
 pt_kickstart_proxy(const smartlist_t *transport_list,
