@@ -613,12 +613,11 @@ test_addr_ip6_helpers(void)
   /* get interface addresses */
   r = get_interface_address6(LOG_DEBUG, AF_INET, &t1);
   i = get_interface_address6(LOG_DEBUG, AF_INET6, &t2);
-#if 0
-  tor_inet_ntop(AF_INET, &t1.sa.sin_addr, buf, sizeof(buf));
-  printf("\nv4 address: %s  (family=%d)", buf, IN_FAMILY(&t1));
-  tor_inet_ntop(AF_INET6, &t2.sa6.sin6_addr, buf, sizeof(buf));
-  printf("\nv6 address: %s  (family=%d)", buf, IN_FAMILY(&t2));
-#endif
+
+  TT_BLATHER(("v4 address: %s (family=%d)", fmt_addr(&t1),
+              tor_addr_family(&t1)));
+  TT_BLATHER(("v6 address: %s (family=%d)", fmt_addr(&t2),
+              tor_addr_family(&t2)));
 
  done:
   ;
